use crate::prelude::*;
use bevy::render::camera::Camera;

pub fn setup(
    mut commands: Commands,
    asset_server: Res<AssetServer>,
    mut materials: ResMut<Assets<ColorMaterial>>,
    map: Res<Map>,
) {
    // Camera
    commands.spawn_bundle(OrthographicCameraBundle::new_2d());

    // Map
    let grass_material = materials.add(asset_server.load("grass.png").into());
    let wall_material = materials.add(asset_server.load("wall.png").into());
    let half_w = MAP_WIDTH * CELL_SIZE / 2;
    let half_h = MAP_HEIGHT * CELL_SIZE / 2;

    for j in 0..MAP_HEIGHT {
        let y = j * CELL_SIZE - half_h;
        for i in 0..MAP_WIDTH {
            let x = i * CELL_SIZE - half_w;
            let index = map.index(i, j);
            let tile = map.tiles[index];
            let material = match tile {
                Tile::Floor => grass_material.clone(),
                Tile::Wall => wall_material.clone(),
            };
            let sprite = Sprite::new(Vec2::new(CELL_SIZE as f32, CELL_SIZE as f32));
            let transform = Transform::from_xyz(x as f32, y as f32, 0.0);

            commands.spawn_bundle(SpriteBundle {
                material,
                sprite,
                transform,
                ..Default::default()
            });
        }
    }
}

pub fn add_player(
    mut commands: Commands,
    asset_server: Res<AssetServer>,
    mut materials: ResMut<Assets<ColorMaterial>>,
) {
    let player_material = materials.add(asset_server.load("marshmallow.png").into());

    commands
        .spawn()
        .insert(Player)
        .insert(Name("Player 1".to_string()))
        .insert_bundle(SpriteBundle {
            material: player_material,
            sprite: Sprite::new(Vec2::new(CELL_SIZE as f32, CELL_SIZE as f32)),
            transform: Transform::from_translation(Vec3::new(0.0, 0.0, 1.0)),
            ..Default::default()
        });
}

pub fn player_movement(
    keyboard_input: Res<Input<KeyCode>>,
    mut query: Query<(&Player, &mut Transform)>,
    time: Res<Time>,
) {
    if let Ok((_, mut transform)) = query.single_mut() {
        let mut direction = Vec2::ZERO;

        if keyboard_input.pressed(KeyCode::A) {
            direction.x -= 1.0;
        }
        if keyboard_input.pressed(KeyCode::D) {
            direction.x += 1.0;
        }
        if keyboard_input.pressed(KeyCode::S) {
            direction.y -= 1.0;
        }
        if keyboard_input.pressed(KeyCode::W) {
            direction.y += 1.0;
        }

        let translation = &mut transform.translation;
        let delta_seconds = time.delta_seconds();

        translation.x += delta_seconds * direction.x * 300.0;
        translation.y += delta_seconds * direction.y * 300.0;
    }
}

pub fn follow_player(
    mut q: QuerySet<(
        Query<&Transform, With<Player>>,
        Query<&mut Transform, With<Camera>>,
    )>,
) {
    if let Ok(player_transform) = q.q0().single() {
        let player_translation = player_transform.translation;

        if let Ok(mut camera_transform) = q.q1_mut().single_mut() {
            let camera_translation = &mut camera_transform.translation;

            camera_translation.x = player_translation.x;
            camera_translation.y = player_translation.y;
        }
    }
}

pub fn greet_player(
    query: Query<&Name, With<Player>>,
    time: Res<Time>,
    mut timer: ResMut<GreetTimer>,
) {
    if timer.0.tick(time.delta()).just_finished() {
        for name in query.iter() {
            println!("Hello, {}!", name.0);
        }
    }
}
